@@ -1,10 +1,8 @@
-/* $Cambridge: exim/src/exim_monitor/em_globals.c,v 1.17 2009/11/16 19:50:36 nm4 Exp $ */
-
 /*************************************************
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2012 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -216,6 +214,9 @@ BOOL    timestamps_utc         = FALSE;
 BOOL    tls_certificate_verified = FALSE;
 uschar *tls_cipher             = NULL;
 uschar *tls_peerdn             = NULL;
+#ifdef SUPPORT_TLS
+uschar *tls_sni                = NULL;
+#endif
 
 tree_node *tree_duplicates     = NULL;
 tree_node *tree_nonrecipients  = NULL;
