@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with retrying unsuccessful deliveries. */
@@ -12,6 +12,54 @@
 
 
 
+/*************************************************
+*         Check the ultimate address timeout     *
+*************************************************/
+
+/* This function tests whether a message has been on the queue longer than
+the maximum retry time for a particular host.
+
+Arguments:
+  host_key      the key to look up a host retry rule
+  domain        the domain to look up a domain retry rule
+  basic_errno   a specific error number, or zero if none
+  more_errno    additional data for the error
+  now           the time
+
+Returns:        TRUE if the ultimate timeout has been reached
+*/
+
+static BOOL
+ultimate_address_timeout(uschar *host_key, uschar *domain, int basic_errno,
+  int more_errno, time_t now)
+{
+BOOL address_timeout = TRUE;   /* no rule => timed out */
+
+retry_config *retry =
+  retry_find_config(host_key+2, domain, basic_errno, more_errno);
+
+if (retry != NULL && retry->rules != NULL)
+  {
+  retry_rule *last_rule;
+  for (last_rule = retry->rules;
+       last_rule->next != NULL;
+       last_rule = last_rule->next);
+  DEBUG(D_transport|D_retry)
+    debug_printf("  received_time=%d diff=%d timeout=%d\n",
+      received_time, (int)(now - received_time), last_rule->timeout);
+  address_timeout = (now - received_time > last_rule->timeout);
+  }
+else
+  {
+  DEBUG(D_transport|D_retry)
+    debug_printf("no retry rule found: assume timed out\n");
+  }
+
+return address_timeout;
+}
+
+
+
 /*************************************************
 *     Set status of a host+address item          *
 *************************************************/
@@ -60,12 +108,12 @@ Returns:    TRUE if the host has expired but is usable because
 */
 
 BOOL
-retry_check_address(char *domain, host_item *host, char *portstring,
-  BOOL include_ip_address, char **retry_host_key, char **retry_message_key)
+retry_check_address(uschar *domain, host_item *host, uschar *portstring,
+  BOOL include_ip_address, uschar **retry_host_key, uschar **retry_message_key)
 {
 BOOL yield = FALSE;
 time_t now = time(NULL);
-char *host_key, *message_key;
+uschar *host_key, *message_key;
 open_db dbblock;
 open_db *dbm_file;
 tree_node *node;
@@ -73,7 +121,7 @@ dbdata_retry *host_retry_record, *message_retry_record;
 
 *retry_host_key = *retry_message_key = NULL;
 
-DEBUG(9) debug_printf("checking status of %s\n", host->name);
+DEBUG(D_transport|D_retry) debug_printf("checking status of %s\n", host->name);
 
 /* Do nothing if status already set; otherwise initialize status as usable. */
 
@@ -100,7 +148,7 @@ the retry database when it is updated). */
 node = tree_search(tree_unusable, host_key);
 if (node != NULL)
   {
-  DEBUG(9) debug_printf("found in tree of unusables\n");
+  DEBUG(D_transport|D_retry) debug_printf("found in tree of unusables\n");
   host->status = (node->data.val > 255)?
     hstatus_unusable_expired : hstatus_unusable;
   host->why = node->data.val & 255;
@@ -110,8 +158,12 @@ if (node != NULL)
 /* Open the retry database, giving up if there isn't one. Otherwise, search for
 the retry records, and then close the database again. */
 
-if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock, FALSE)) == NULL)
+if ((dbm_file = dbfn_open(US"retry", O_RDONLY, &dbblock, FALSE)) == NULL)
+  {
+  DEBUG(D_deliver|D_retry|D_hints_lookup)
+    debug_printf("no retry data available\n");
   return FALSE;
+  }
 host_retry_record = dbfn_read(dbm_file, host_key);
 message_retry_record = dbfn_read(dbm_file, message_key);
 dbfn_close(dbm_file);
@@ -120,22 +172,22 @@ dbfn_close(dbm_file);
 
 if (host_retry_record == NULL)
   {
-  DEBUG(9) debug_printf("no host retry record\n");
+  DEBUG(D_transport|D_retry) debug_printf("no host retry record\n");
   }
 else if (now - host_retry_record->time_stamp > retry_data_expire)
   {
   host_retry_record = NULL;
-  DEBUG(9) debug_printf("host retry record too old\n");
+  DEBUG(D_transport|D_retry) debug_printf("host retry record too old\n");
   }
 
 if (message_retry_record == NULL)
   {
-  DEBUG(9) debug_printf("no message retry record\n");
+  DEBUG(D_transport|D_retry) debug_printf("no message retry record\n");
   }
 else if (now - message_retry_record->time_stamp > retry_data_expire)
   {
   message_retry_record = NULL;
-  DEBUG(9) debug_printf("message retry record too old\n");
+  DEBUG(D_transport|D_retry) debug_printf("message retry record too old\n");
   }
 
 /* If there's a host-specific retry record, check for reaching the retry
@@ -147,42 +199,31 @@ host. */
 
 if (host_retry_record != NULL)
   {
-  time_t now = time(NULL);
-
   *retry_host_key = host_key;
 
+  /* We have not reached the next try time. Check for the ultimate address
+  timeout if the host has not expired. */
+
   if (now < host_retry_record->next_try && !deliver_force)
     {
-    /* Check for ultimate address timeout if host not expired */
-
-    if (!host_retry_record->expired)
+    DEBUG(D_transport|D_retry)
       {
-      BOOL address_timeout = FALSE;
+      debug_printf("host retry time not reached: checking ultimate address "
+        "timeout\n");
+      debug_printf("  now=%d first_failed=%d next_try=%d expired=%d\n",
+        (int)now, (int)host_retry_record->first_failed,
+        (int)host_retry_record->next_try,
+        host_retry_record->expired);
+      }
 
-      retry_config *retry =
-        retry_find_config(host_key+2, domain,
-          host_retry_record->basic_errno,
-          host_retry_record->more_errno);
-
-      DEBUG(9) debug_printf("retry time not reached: checking ultimate "
-        "address timeout\n");
-
-      if (retry != NULL)
-        {
-        retry_rule *last_rule;
-        for (last_rule = retry->rules;
-             last_rule->next != NULL;
-             last_rule = last_rule->next);
-        if (now - received_time > last_rule->timeout) address_timeout = TRUE;
-        }
-      else address_timeout = TRUE;    /* No rule => timed out */
-
-      if (address_timeout)
-        {
-        DEBUG(5) debug_printf("on queue longer than maximum retry for "
+    if (!host_retry_record->expired &&
+        ultimate_address_timeout(host_key, domain,
+          host_retry_record->basic_errno, host_retry_record->more_errno, now))
+      {
+      DEBUG(D_transport|D_retry)
+        debug_printf("on queue longer than maximum retry for "
           "address - allowing delivery\n");
-        return FALSE;
-        }
+      return FALSE;
       }
 
     /* We have not hit the ultimate address timeout; host is unusable. */
@@ -200,15 +241,33 @@ if (host_retry_record != NULL)
   }
 
 /* It's OK to try the host. If there's a message-specific retry record, check
-for reaching its retry time (or forcing). If not, mark the host unusable. */
+for reaching its retry time (or forcing). If not, mark the host unusable,
+unless the ultimate address timeout has been reached. */
 
 if (message_retry_record != NULL)
   {
   *retry_message_key = message_key;
-  if (time(NULL) < message_retry_record->next_try && !deliver_force)
+  if (now < message_retry_record->next_try && !deliver_force)
     {
-    host->status = hstatus_unusable;
-    host->why = hwhy_retry;
+    DEBUG(D_transport|D_retry)
+      {
+      debug_printf("host+message retry time not reached: checking ultimate "
+        "address timeout\n");
+      debug_printf("  now=%d first_failed=%d next_try=%d expired=%d\n",
+        (int)now, (int)message_retry_record->first_failed,
+        (int)message_retry_record->next_try, message_retry_record->expired);
+      }
+    if (!ultimate_address_timeout(host_key, domain, 0, 0, now))
+      {
+      host->status = hstatus_unusable;
+      host->why = hwhy_retry;
+      }
+    else
+      {
+      DEBUG(D_transport|D_retry)
+        debug_printf("on queue longer than maximum retry for "
+          "address - allowing delivery\n");
+      }
     return FALSE;
     }
   }
@@ -223,47 +282,51 @@ return yield;
 *           Add a retry item to an address       *
 *************************************************/
 
-/* Retry items are chained onto an address when it is deferred either by
-a director, router, or transport, or if it succeeds or fails and there was a
-previous retry item that now needs to be deleted. Sometimes there can be both
-kinds of item: for example, if routing was deferred but then succeeded, and
-delivery then deferred. In that case there is a delete item for the routing
-retry, and an updating item for the delivery.
+/* Retry items are chained onto an address when it is deferred either by router
+or by a transport, or if it succeeds or fails and there was a previous retry
+item that now needs to be deleted. Sometimes there can be both kinds of item:
+for example, if routing was deferred but then succeeded, and delivery then
+deferred. In that case there is a delete item for the routing retry, and an
+updating item for the delivery.
+
+(But note that that is only visible at the outer level, because in remote
+delivery subprocesses, the address starts "clean", with no retry items carried
+in.)
 
 These items are used at the end of a delivery attempt to update the retry
-database. The key name is built from a type letter and the domain, optionally
-with "<local_part>@" on the front - this is used for local destinations - or
-passed in externally for remote destinations, which might be more complex (SMTP
-destinations consist of the domain + the IP address).
+database. The keys start R: for routing delays and T: for transport delays.
 
 Arguments:
-  addr    the address block
-  type    the type letter: one of 'D', 'R', or 'T'
-  use_at  if TRUE, "addr->local_part@" is included after "type:"
-  key     if NULL, addr->domain is used, otherwise key
+  addr    the address block onto which to hang the item
+  key     the retry key
   flags   delete, host, and message flags, copied into the block
 
 Returns:  nothing
 */
 
 void
-retry_add_item(address_item *addr, char *type, BOOL use_at, char *key,
-  int flags)
+retry_add_item(address_item *addr, uschar *key, int flags)
 {
 retry_item *rti = store_get(sizeof(retry_item));
 rti->next = addr->retries;
 addr->retries = rti;
-rti->destination = string_sprintf("%s:%s%s%s", type,
-  use_at? addr->local_part : "",
-  use_at? "@" : "",
-  (key != NULL)? key : addr->domain);
+rti->key = key;
 rti->basic_errno = addr->basic_errno;
 rti->more_errno = addr->more_errno;
 rti->message = addr->message;
 rti->flags = flags;
 
-DEBUG(7) debug_printf("added retry item for %s: errno=%d %d flags=%d\n",
-  rti->destination, rti->basic_errno, rti->more_errno, flags);
+DEBUG(D_transport|D_retry)
+  {
+  int letter = rti->more_errno & 255;
+  debug_printf("added retry item for %s: errno=%d more_errno=", rti->key,
+    rti->basic_errno);
+  if (letter == 'A' || letter == 'M')
+    debug_printf("%d,%c", (rti->more_errno >> 8) & 255, letter);
+  else
+    debug_printf("%d", rti->more_errno);
+  debug_printf(" flags=%d\n", flags);
+  }
 }
 
 
@@ -273,61 +336,84 @@ DEBUG(7) debug_printf("added retry item for %s: errno=%d %d flags=%d\n",
 *************************************************/
 
 /* Search the in-store retry information for the first retry item that applies
-to a given destination. Remote Internet destinations are given as
-domain:address. It doesn't seem necessary to go to that level of detail in
-retry rules, so we chop off the address and match on the domain part only.
-Local addresses are given as user@domain. The alternative destination is the
-domain name, which is always just a domain. This function is used for real
-only in this module, but it is global so it can be tested by means of the -brt
-option.
+to a given destination. If the key contains an @ we are probably handling a
+local delivery and have a complete address to search for; this happens when
+retry_use_local_part is set on a router. Otherwise, the key is likely to be a
+host name for a remote delivery, or a domain name for a local delivery. We
+prepend *@ on the front of it so that it will match a retry item whose address
+item pattern is independent of the local part. The alternate key, if set, is
+always just a domain, so we treat it likewise.
 
 Arguments:
-  destination  destination for which retry info is wanted
-  alternate    alternative destination, always just a domain
+  key          key for which retry info is wanted
+  alternate    alternative key, always just a domain
   basic_errno  specific error predicate on the retry rule, or zero
-  more_errno   additional data for errno predicate (for errno_quota only)
+  more_errno   additional data for errno predicate
 
 Returns:       pointer to retry rule, or NULL
 */
 
 retry_config *
-retry_find_config(char *destination, char *alternate, int basic_errno,
+retry_find_config(uschar *key, uschar *alternate, int basic_errno,
   int more_errno)
 {
-int domain = 0;
-int replace = ':';
-int sep = UCHAR_MAX + 1;
-char *colon = strchr(destination, ':');
+int replace = 0;
+uschar *use_key, *use_alternate;
+uschar *colon = Ustrchr(key, ':');
 retry_config *yield;
 
-/* If there's a colon in the destination, temporarily replace it with
-a zero to terminate the string there. If there's no colon there should be
-an @; set up the domain offset. */
+/* If there's a colon in the key, there are two possibilities:
 
-if (colon == NULL)
+(1) This is a key for a host, ip address, and possibly port, in the format
+
+      hostname:ip+port
+
+    In this case, we temporarily replace the colon with a zero, to terminate
+    the string after the host name.
+
+(2) This is a key for a pipe, file, or autoreply delivery, in the format
+
+      pipe-or-file-or-auto:x@y
+
+    where x@y is the original address that provoked the delivery. The pipe or
+    file or auto will start with | or / or >, whereas a host name will start
+    with a letter or a digit. In this case we want to use the original address
+    to search for a retry rule. */
+
+if (colon != NULL)
   {
-  char *domain_offset = strrchr(destination, '@');
-  if (domain_offset != NULL) domain = domain_offset - destination + 1;
-  colon = destination + (int)strlen(destination);
-  replace = 0;
+  if (isalnum(*key))
+    replace = ':';
+  else
+    key = Ustrrchr(key, ':') + 1;   /* Take from the last colon */
   }
+
+if (replace == 0) colon = key + Ustrlen(key);
 *colon = 0;
 
+/* Sort out the keys */
+
+use_key = (Ustrchr(key, '@') != NULL)? key : string_sprintf("*@%s", key);
+use_alternate = (alternate == NULL)? NULL : string_sprintf("*@%s", alternate);
+
 /* Scan the configured retry items. */
 
 for (yield = retries; yield != NULL; yield = yield->next)
   {
-  char *dlist = yield->destination;
+  uschar *plist = yield->pattern;
+  uschar *slist = yield->senders;
 
-  /* If a specific error is set for this item, check that we are
-  handling that specific error, and if so, check any additional
-  error information if required. Special code is required for quota
-  errors, as these can either be system quota errors, or Exim's own
-  quota imposition, which has a different error number. Full partitions
-  are also treated in the same way as quota errors. */
+  /* If a specific error is set for this item, check that we are handling that
+  specific error, and if so, check any additional error information if
+  required. */
 
   if (yield->basic_errno != 0)
     {
+    /* Special code is required for quota errors, as these can either be system
+    quota errors, or Exim's own quota imposition, which has a different error
+    number. Full partitions are also treated in the same way as quota errors.
+    */
+
     if (yield->basic_errno == ERRNO_EXIMQUOTA)
       {
       if ((basic_errno != ERRNO_EXIMQUOTA && basic_errno != errno_quota &&
@@ -335,6 +421,66 @@ for (yield = retries; yield != NULL; yield = yield->next)
           (yield->more_errno != 0 && yield->more_errno > more_errno))
         continue;
       }
+
+    /* The TLSREQUIRED error also covers TLSFAILURE. These are subtly different
+    errors, but not worth separating at this level. */
+
+    else if (yield->basic_errno == ERRNO_TLSREQUIRED)
+      {
+      if (basic_errno != ERRNO_TLSREQUIRED && basic_errno != ERRNO_TLSFAILURE)
+        continue;
+      }
+
+    /* Handle 4xx responses to MAIL, RCPT, or DATA. The code that was received
+    is in the 2nd least significant byte of more_errno (with 400 subtracted).
+    The required value is coded in the 2nd least significant byte of the
+    yield->more_errno field as follows:
+
+      255     => any 4xx code
+      >= 100  => the decade must match the value less 100
+      < 100   => the exact value must match
+    */
+
+    else if (yield->basic_errno == ERRNO_MAIL4XX ||
+             yield->basic_errno == ERRNO_RCPT4XX ||
+             yield->basic_errno == ERRNO_DATA4XX)
+      {
+      int wanted;
+      if (basic_errno != yield->basic_errno) continue;
+      wanted = (yield->more_errno >> 8) & 255;
+      if (wanted != 255)
+        {
+        int evalue = (more_errno >> 8) & 255;
+        if (wanted >= 100)
+          {
+          if ((evalue/10)*10 != wanted - 100) continue;
+          }
+        else if (evalue != wanted) continue;
+        }
+      }
+
+    /* There are some special cases for timeouts */
+
+    else if (yield->basic_errno == ETIMEDOUT)
+      {
+      if (basic_errno != ETIMEDOUT) continue;
+
+      /* Just RTEF_CTOUT in the rule => don't care about 'A'/'M' addresses */
+      if (yield->more_errno == RTEF_CTOUT)
+        {
+        if ((more_errno & RTEF_CTOUT) == 0) continue;
+        }
+
+      else if (yield->more_errno != 0)
+        {
+        int cf_errno = more_errno;
+        if ((yield->more_errno & RTEF_CTOUT) == 0) cf_errno &= ~RTEF_CTOUT;
+        if (yield->more_errno != cf_errno) continue;
+        }
+      }
+
+    /* Default checks for exact match */
+
     else
       {
       if (yield->basic_errno != basic_errno ||
@@ -343,18 +489,23 @@ for (yield = retries; yield != NULL; yield = yield->next)
       }
     }
 
-  /* If the destination contains an @ we are handling a local delivery;
-  otherwise it's a remote host name. Either way, we can persuade the
-  match_address function to handle it. */
+  /* If the "senders" condition is set, check it. Note that sender_address may
+  be null during -brt checking, in which case we do not use this rule. */
 
-  if (match_address_list(destination, domain, TRUE, &dlist, -1, &sep, FALSE))
+  if (slist != NULL && (sender_address == NULL ||
+      match_address_list(sender_address, TRUE, TRUE, &slist, NULL, -1, 0,
+        NULL) != OK))
+    continue;
+
+  /* Check for a match between the address list item at the start of this retry
+  rule and either the main or alternate keys. */
+
+  if (match_address_list(use_key, TRUE, TRUE, &plist, NULL, -1, UCHAR_MAX+1,
+        NULL) == OK ||
+     (use_alternate != NULL &&
+      match_address_list(use_alternate, TRUE, TRUE, &plist, NULL, -1,
+        UCHAR_MAX+1, NULL) == OK))
     break;
-
-  /* If the match failed, have a go at matching on the alternative destination,
-  which is normally the domain as opposed to the host for a remote delivery. */
-
-  if (alternate != NULL &&
-      match_address_list(alternate, 0, TRUE, &dlist, -1, &sep, FALSE)) break;
   }
 
 *colon = replace;
@@ -396,7 +547,7 @@ open_db *dbm_file = NULL;
 time_t now = time(NULL);
 int i;
 
-DEBUG(2) debug_printf("Processing retry items\n");
+DEBUG(D_retry) debug_printf("Processing retry items\n");
 
 /* Three-times loop to handle succeeded, failed, and deferred addresses.
 Deferred addresses must be handled after failed ones, because some may be moved
@@ -410,8 +561,8 @@ for (i = 0; i < 3; i++)
     (i==1)? addr_failed : addr_defer;
   address_item **saved_paddr = NULL;
 
-  DEBUG(5) debug_printf("%s addresses:\n", (i == 0)? "Succeeded" : (i == 1)?
-    "Failed" : "Deferred");
+  DEBUG(D_retry) debug_printf("%s addresses:\n", (i == 0)? "Succeeded" :
+    (i == 1)? "Failed" : "Deferred");
 
   /* Loop for each address on the chain. For deferred addresses, the whole
   address times out unless one of its retry addresses has a retry rule that
@@ -433,14 +584,14 @@ for (i = 0; i < 3; i++)
       int update_count = 0;
       int timedout_count = 0;
 
-      DEBUG(5) debug_printf("%s%s\n", addr->orig, (addr->retries == NULL)?
+      DEBUG(D_retry) debug_printf("%s%s\n", addr->address, (addr->retries == NULL)?
         ": no retry items" : "");
 
       /* Loop for each retry item. */
 
       for (rti = addr->retries; rti != NULL; rti = rti->next)
         {
-        char *message;
+        uschar *message;
         int message_length, message_space, failing_interval, next_try;
         retry_rule *rule, *final_rule;
         retry_config *retry;
@@ -453,8 +604,14 @@ for (i = 0; i < 3; i++)
         reached their retry next try time. */
 
         if (dbm_file == NULL)
-          dbm_file = dbfn_open("retry", O_RDWR|O_CREAT, &dbblock, TRUE);
-        if (dbm_file == NULL) return;
+          dbm_file = dbfn_open(US"retry", O_RDWR, &dbblock, TRUE);
+
+        if (dbm_file == NULL)
+          {
+          DEBUG(D_deliver|D_retry|D_hints_lookup)
+            debug_printf("retry database not available for updating\n");
+          return;
+          }
 
         /* If there are no deferred addresses, that is, if this message is
         completing, and the retry item is for a message-specific SMTP error,
@@ -472,9 +629,9 @@ for (i = 0; i < 3; i++)
 
         if ((rti->flags & rf_delete) != 0)
           {
-          (void)dbfn_delete(dbm_file, rti->destination);
-          DEBUG(5) debug_printf("deleted retry information for %s\n",
-            rti->destination);
+          (void)dbfn_delete(dbm_file, rti->key);
+          DEBUG(D_retry)
+            debug_printf("deleted retry information for %s\n", rti->key);
           continue;
           }
 
@@ -490,42 +647,43 @@ for (i = 0; i < 3; i++)
         information is found, we can't generate a retry time, so there is
         no point updating the database. This retry item is timed out. */
 
-        if ((retry = retry_find_config(rti->destination+2,
+        if ((retry = retry_find_config(rti->key + 2,
              ((rti->flags & rf_host) != 0)? addr->domain : NULL,
              rti->basic_errno, rti->more_errno)) == NULL)
           {
-          DEBUG(5) debug_printf("No configured retry item for %s%s%s\n",
-            rti->destination,
-            ((rti->flags & rf_host) != 0)? " or " : "",
-            ((rti->flags & rf_host) != 0)? addr->domain : "");
+          DEBUG(D_retry) debug_printf("No configured retry item for %s%s%s\n",
+            rti->key,
+            ((rti->flags & rf_host) != 0)? US" or " : US"",
+            ((rti->flags & rf_host) != 0)? addr->domain : US"");
           if (addr == endaddr) timedout_count++;
           continue;
           }
 
-        DEBUG(5)
+        DEBUG(D_retry)
           {
           if ((rti->flags & rf_host) != 0)
-            debug_printf("retry for %s (%s) = %s\n", rti->destination,
-              addr->domain, retry->destination);
+            debug_printf("retry for %s (%s) = %s %d %d\n", rti->key,
+              addr->domain, retry->pattern, retry->basic_errno,
+              retry->more_errno);
           else
-            debug_printf("retry for %s = %s\n", rti->destination,
-              retry->destination);
+            debug_printf("retry for %s = %s %d %d\n", rti->key, retry->pattern,
+              retry->basic_errno, retry->more_errno);
           }
 
         /* Set up the message for the database retry record. Because DBM
         records have a maximum data length, we enforce a limit. There isn't
         much point in keeping a huge message here, anyway. */
 
-        message = (rti->basic_errno > 0)? strerror(rti->basic_errno) :
+        message = (rti->basic_errno > 0)? US strerror(rti->basic_errno) :
           (rti->message == NULL)?
-          "unknown error" : string_printing(rti->message);
-        message_length = strlen(message);
-        if (message_length > 100) message_length = 100;
+          US"unknown error" : string_printing(rti->message);
+        message_length = Ustrlen(message);
+        if (message_length > 150) message_length = 150;
 
         /* Read a retry record from the database or construct a new one.
         Ignore an old one if it is too old since it was last updated. */
 
-        retry_record = dbfn_read(dbm_file, rti->destination);
+        retry_record = dbfn_read(dbm_file, rti->key);
         if (retry_record != NULL &&
             now - retry_record->time_stamp > retry_data_expire)
           retry_record = NULL;
@@ -540,11 +698,22 @@ for (i = 0; i < 3; i++)
           retry_record->expired = FALSE;
           retry_record->text[0] = 0;      /* just in case */
           }
-        else message_space = (int)strlen(retry_record->text);
+        else message_space = Ustrlen(retry_record->text);
 
         /* Compute how long this destination has been failing */
 
         failing_interval = now - retry_record->first_failed;
+        DEBUG(D_retry) debug_printf("failing_interval=%d message_age=%d\n",
+          failing_interval, message_age);
+
+        /* For a non-host error, if the message has been on the queue longer
+        than the recorded time of failure, use the message's age instead. This
+        can happen when some messages can be delivered and others cannot; a
+        successful delivery will reset the first_failed time, and this can lead
+        to a failing message being retried too often. */
+
+        if ((rti->flags & rf_host) == 0 && message_age > failing_interval)
+          failing_interval = message_age;
 
         /* Search for the current retry rule. The cutoff time of the
         last rule is handled differently to the others. The rule continues
@@ -599,9 +768,16 @@ for (i = 0; i < 3; i++)
 
         This implements "timeout this rule if EITHER the host (or routing or
         directing) has been failing for more than the maximum time, OR if the
-        message has been on the queue for more than the maximum time." */
+        message has been on the queue for more than the maximum time."
 
-        if (received_time < retry_record->first_failed &&
+        February 2006: It is possible that this code is no longer needed
+        following the change to the retry calculation to use the message age if
+        it is larger than the time since first failure. It may be that the
+        expired flag is always set when the other conditions are met. However,
+        this is a small bit of code, and it does no harm to leave it in place,
+        just in case. */
+
+        if (received_time <= retry_record->first_failed &&
             addr == endaddr && !retry_record->expired && rule != NULL)
           {
           retry_rule *last_rule;
@@ -610,7 +786,7 @@ for (i = 0; i < 3; i++)
                last_rule = last_rule->next);
           if (now - received_time > last_rule->timeout)
             {
-            DEBUG(5) debug_printf("on queue longer than maximum retry\n");
+            DEBUG(D_retry) debug_printf("on queue longer than maximum retry\n");
             timedout_count++;
             rule = NULL;
             }
@@ -626,15 +802,25 @@ for (i = 0; i < 3; i++)
         if (rule == NULL) next_try = now; else
           {
           if (rule->rule == 'F') next_try = now + rule->p1;
-          else  /* assume rule = 'G' */
+          else  /* rule = 'G' or 'H' */
             {
             int last_predicted_gap =
               retry_record->next_try - retry_record->last_try;
             int last_actual_gap = now - retry_record->last_try;
             int lastgap = (last_predicted_gap < last_actual_gap)?
               last_predicted_gap : last_actual_gap;
-            next_try = now + ((lastgap < rule->p1)? rule->p1 :
-               (lastgap * rule->p2)/1000);
+            int next_gap = (lastgap * rule->p2)/1000;
+            if (rule->rule == 'G')
+              {
+              next_try = now + ((lastgap < rule->p1)? rule->p1 : next_gap);
+              }
+            else  /* The 'H' rule */
+              {
+              next_try = now + rule->p1;
+              if (next_gap > rule->p1)
+                next_try += random_number(next_gap - rule->p1)/2 +
+                  (next_gap - rule->p1)/2;
+              }
             }
           }
 
@@ -660,38 +846,51 @@ for (i = 0; i < 3; i++)
         retry_record->next_try = next_try;
         retry_record->basic_errno = rti->basic_errno;
         retry_record->more_errno = rti->more_errno;
-        strncpy(retry_record->text, message, message_length);
+        Ustrncpy(retry_record->text, message, message_length);
         retry_record->text[message_length] = 0;
 
-        DEBUG(5)
+        DEBUG(D_retry)
           {
-          debug_printf("Writing retry data for %s\n", rti->destination);
+          int letter = retry_record->more_errno & 255;
+          debug_printf("Writing retry data for %s\n", rti->key);
           debug_printf("  first failed=%d last try=%d next try=%d expired=%d\n",
-            retry_record->first_failed, retry_record->last_try,
-            retry_record->next_try, retry_record->expired);
-          debug_printf("  error %d %d: %s\n", retry_record->basic_errno,
-            retry_record->more_errno, retry_record->text);
+            (int)retry_record->first_failed, (int)retry_record->last_try,
+            (int)retry_record->next_try, retry_record->expired);
+          debug_printf("  errno=%d more_errno=", retry_record->basic_errno);
+          if (letter == 'A' || letter == 'M')
+            debug_printf("%d,%c", (retry_record->more_errno >> 8) & 255,
+              letter);
+          else
+            debug_printf("%d", retry_record->more_errno);
+          debug_printf(" %s\n", retry_record->text);
           }
 
-        (void)dbfn_write(dbm_file, rti->destination, retry_record,
+        (void)dbfn_write(dbm_file, rti->key, retry_record,
           sizeof(dbdata_retry) + message_length);
         }                            /* Loop for each retry item */
 
       /* If all the non-delete retry items are timed out, the address is
-      timed out. */
+      timed out, provided that we didn't skip any hosts because their retry
+      time was not reached (or because of hosts_max_try). */
 
       if (update_count > 0 && update_count == timedout_count)
         {
-        DEBUG(5) debug_printf("timed out: all retries expired\n");
-        timed_out = TRUE;
+        if (!testflag(endaddr, af_retry_skipped))
+          {
+          DEBUG(D_retry) debug_printf("timed out: all retries expired\n");
+          timed_out = TRUE;
+          }
+        else
+          {
+          DEBUG(D_retry)
+            debug_printf("timed out but some hosts were skipped\n");
+          }
         }
-      }                              /* Loop for address & parents */
+      }     /* Loop for an address and its parents */
 
     /* If this is a deferred address, and retry processing was requested by
-    means of one or more retry items, and no actions were skipped because a
-    retry time had not been reached, and no retry item is still active,
-    move the address to the failed queue, and restart this loop without
-    updating paddr.
+    means of one or more retry items, and they all timed out, move the address
+    to the failed queue, and restart this loop without updating paddr.
 
     If there were several addresses batched in the same remote delivery, only
     the original top one will have host retry items attached to it, but we want
@@ -702,8 +901,7 @@ for (i = 0; i < 3; i++)
 
     if (i == 2)   /* Handling defers */
       {
-      if (endaddr->retries != NULL && !testflag(endaddr, af_retry_skipped) &&
-          timed_out)
+      if (endaddr->retries != NULL && timed_out)
         {
         if (last_first == endaddr) paddr = saved_paddr;
         addr = *paddr;
@@ -715,13 +913,16 @@ for (i = 0; i < 3; i++)
         for (;; addr = addr->next)
           {
           setflag(addr, af_retry_timedout);
-          addr->message = (addr->message == NULL)? "retry timeout exceeded" :
+          addr->message = (addr->message == NULL)? US"retry timeout exceeded" :
             string_sprintf("%s: retry timeout exceeded", addr->message);
+          addr->user_message = (addr->user_message == NULL)?
+            US"retry timeout exceeded" :
+            string_sprintf("%s: retry timeout exceeded", addr->user_message);
           log_write(0, LOG_MAIN, "** %s%s%s%s: retry timeout exceeded",
-            addr->orig,
-           (addr->parent == NULL)? "" : " <",
-           (addr->parent == NULL)? "" : addr->parent->orig,
-           (addr->parent == NULL)? "" : ">");
+            addr->address,
+           (addr->parent == NULL)? US"" : US" <",
+           (addr->parent == NULL)? US"" : addr->parent->address,
+           (addr->parent == NULL)? US"" : US">");
 
           if (addr == endaddr) break;
           }
@@ -750,7 +951,7 @@ for (i = 0; i < 3; i++)
 
 if (dbm_file != NULL) dbfn_close(dbm_file);
 
-DEBUG(2) debug_printf("end of retry processing\n");
+DEBUG(D_retry) debug_printf("end of retry processing\n");
 }
 
 /* End of retry.c */
